/*
 * Copyright 2016-2023 NXP
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * o Redistributions of source code must retain the above copyright notice, this list
 *   of conditions and the following disclaimer.
 *
 * o Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * o Neither the name of NXP Semiconductor, Inc. nor the names of its
 *   contributors may be used to endorse or promote products derived from this
 *   software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file    LPC55S28_Project_FT.cpp
 * @brief   Application entry point.
 */

#define CODEC_WM8904_ENABLE	1

#include <stdio.h>
#include "board.h"
#include "peripherals.h"
#include "pin_mux.h"
#include "clock_config.h"
#include "LPC55S28.h"
#include "fsl_debug_console.h"
#include "fsl_wm8904.h"
#include "fsl_codec_common.h"
#include "fsl_adapter_audio.h"
#include "dsp.hpp"
#include "app.h"
#include "lcd.h"
#include "grafika.h"
#include "gui.h"
/* TODO: insert other include files here. */

/* TODO: insert other definitions and declarations here. */
typedef uint8_t BYTE;

#define DEMO_I2S_MASTER_CLOCK_FREQUENCY (24576000)

codec_handle_t codecHandle;
wm8904_config_t wm8904Config = {
	.master             = false,
	.sysClkSource		= kWM8904_SysClkSourceMCLK,
    .protocol           = kWM8904_ProtocolI2S,
    .format             = {.sampleRate = kWM8904_SampleRate48kHz, .bitWidth = kWM8904_BitWidth16},
    .mclk_HZ            = DEMO_I2S_MASTER_CLOCK_FREQUENCY,
    .recordSource = kWM8904_RecordSourceLineInput,
    .recordChannelLeft  = kWM8904_RecordChannelLeft2,
    .recordChannelRight = kWM8904_RecordChannelRight2,
    .playSource         = kWM8904_PlaySourceDAC,
    .slaveAddress       = WM8904_I2C_ADDRESS,
    .i2cConfig    = {.codecI2CInstance = BOARD_CODEC_I2C_INSTANCE, .codecI2CSourceClock = BOARD_CODEC_I2C_CLOCK_FREQ},
};
codec_config_t boardCodecConfig = {.codecDevType = kCODEC_WM8904, .codecDevConfig = &wm8904Config};

hal_audio_status_t st, st2, st3;

int16_t WartoscWE_L = 0;
int16_t WartoscWE_P = 0;
int16_t WartoscWY_L = 0;
int16_t WartoscWY_P = 0;

void BOARD_Codec_Init()
{
	status_t sc;

	SYSCON->MCLKIO = 1;

    if ((sc = CODEC_Init(&codecHandle, &boardCodecConfig)) != kStatus_Success)
    {
        assert(false);
    }

    uint16_t w;
    WM8904_ReadRegister((wm8904_handle_t *)&codecHandle.codecDevHandle, WM8904_CLK_RATES_0, &w);

    CODEC_SetPower(&codecHandle, kCODEC_ModuleLinein, true);
    CODEC_SetPower(&codecHandle, kCODEC_ModuleHeadphone, true);
    CODEC_SetPower(&codecHandle, kCODEC_ModulePGA, true);

    if (CODEC_SetVolume(&codecHandle, kCODEC_VolumeHeadphoneLeft | kCODEC_VolumeHeadphoneRight, 99) !=
        kStatus_Success)
    {
        assert(false);
    }

}

/*
 * @brief   Application entry point.
 */



class bar_eq : public KBar
{
public:
	virtual void CallbackBar(int id, bool kierunek)
	{
		Poz.Tab[(int)Id] = (float)this->Wartosc/100;
	}
};

class radio_eq : public KRadio
{
public:
	virtual void CallbackRadio(int id)
	{
		if(Wartosc ==0)
		{
			WM8904_ModifyRegister((wm8904_handle_t *)codecHandle.codecDevHandle, WM8904_ANALOG_LEFT_IN_0, 0x1F, 5);
			WM8904_ModifyRegister((wm8904_handle_t *)codecHandle.codecDevHandle, WM8904_ANALOG_RIGHT_IN_0, 0x1F, 5);
		} else
		{
			WM8904_ModifyRegister((wm8904_handle_t *)codecHandle.codecDevHandle, WM8904_ANALOG_LEFT_IN_0, 0x1F, 0x1F);
			WM8904_ModifyRegister((wm8904_handle_t *)codecHandle.codecDevHandle, WM8904_ANALOG_RIGHT_IN_0, 0x1F, 0x1F);
		}

	}
};





void UruchomGUI()
{
    LCDImp::Inicjuj();

    bar_eq bar_l20, bar_l50, bar_l100, bar_l200, bar_l500, bar_l1K, bar_l2K, bar_l5K, bar_l10K;
    bar_eq bar_p20, bar_p50, bar_p100, bar_p200, bar_p500, bar_p1K, bar_p2K, bar_p5K, bar_p10K;

	KLabel tkl,tkp;
	KLabel lab_l20, lab_l50, lab_l100, lab_l200, lab_l500, lab_l1K, lab_l2K, lab_l5K, lab_l10K;
	KLabel lab_p20, lab_p50, lab_p100, lab_p200, lab_p500, lab_p1K, lab_p2K, lab_p5K, lab_p10K;

/********************************************** kanał lewy *********************************************************************************/

	tkl.Tekst = "LEWY";
    tkl.X1 = 0;
    tkl.Y1 = 20;
    tkl.Pion = true;
    Gui.WstawKontrolke(&tkl);

    bar_l20.X1 = 10;
    bar_l20.X2 = 20;
    bar_l20.Y1 = 10;
    bar_l20.Y2 = 63;
    bar_l20.Wartosc = 50;
    bar_l20.Id = KL_20;
    Gui.WstawKontrolke(&bar_l20);
    lab_l20.X1 = 7;
    lab_l20.Y1 = 0;
    lab_l20.Tekst = "20";
    Gui.WstawKontrolke(&lab_l20);

    bar_l50.X1 = 30;
    bar_l50.X2 = 40;
    bar_l50.Y1 = 10;
    bar_l50.Y2 = 63;
    bar_l50.Wartosc = 50;
    bar_l50.Id = KL_50;
    Gui.WstawKontrolke(&bar_l50);
    lab_l50.X1 = 27;
    lab_l50.Y1 = 0;
    lab_l50.Tekst = "50";
    Gui.WstawKontrolke(&lab_l50);

    bar_l100.X1 = 50;
    bar_l100.X2 = 60;
    bar_l100.Y1 = 10;
    bar_l100.Y2 = 63;
    bar_l100.Wartosc = 50;
    bar_l100.Id = KL_100;
    Gui.WstawKontrolke(&bar_l100);
    lab_l100.X1 = 47;
    lab_l100.Y1 = 0;
    lab_l100.Tekst = "1D";
    Gui.WstawKontrolke(&lab_l100);

    bar_l200.X1 = 70;
    bar_l200.X2 = 80;
    bar_l200.Y1 = 10;
    bar_l200.Y2 = 63;
    bar_l200.Wartosc = 50;
    bar_l200.Id = KL_200;
    Gui.WstawKontrolke(&bar_l200);
    lab_l200.X1 = 67;
    lab_l200.Y1 = 0;
    lab_l200.Tekst = "2D";
    Gui.WstawKontrolke(&lab_l200);

    bar_l500.X1 = 90;
    bar_l500.X2 = 100;
    bar_l500.Y1 = 10;
    bar_l500.Y2 = 63;
    bar_l500.Wartosc = 50;
    bar_l500.Id = KL_500;
    Gui.WstawKontrolke(&bar_l500);
    lab_l500.X1 = 87;
    lab_l500.Y1 = 0;
    lab_l500.Tekst = "5D";
    Gui.WstawKontrolke(&lab_l500);

    bar_l1K.X1 = 110;
    bar_l1K.X2 = 120;
    bar_l1K.Y1 = 10;
    bar_l1K.Y2 = 63;
    bar_l1K.Wartosc = 50;
    bar_l1K.Id = KL_1K;
    Gui.WstawKontrolke(&bar_l1K);
    lab_l1K.X1 = 107;
    lab_l1K.Y1 = 0;
    lab_l1K.Tekst = "1K";
    Gui.WstawKontrolke(&lab_l1K);

    bar_l2K.X1 = 130;
    bar_l2K.X2 = 140;
    bar_l2K.Y1 = 10;
    bar_l2K.Y2 = 63;
    bar_l2K.Wartosc = 50;
    bar_l2K.Id = KL_2K;
    Gui.WstawKontrolke(&bar_l2K);
    lab_l2K.X1 = 127;
    lab_l2K.Y1 = 0;
    lab_l2K.Tekst = "2K";
    Gui.WstawKontrolke(&lab_l2K);

    bar_l5K.X1 = 150;
    bar_l5K.X2 = 160;
    bar_l5K.Y1 = 10;
    bar_l5K.Y2 = 63;
    bar_l5K.Wartosc = 50;
    bar_l5K.Id = KL_5K;
    Gui.WstawKontrolke(&bar_l5K);
    lab_l5K.X1 = 147;
    lab_l5K.Y1 = 0;
    lab_l5K.Tekst = "5K";
    Gui.WstawKontrolke(&lab_l5K);
    //Gui.RysujKontrolki();

    bar_l10K.X1 = 170;
    bar_l10K.X2 = 180;
    bar_l10K.Y1 = 10;
    bar_l10K.Y2 = 63;
    bar_l10K.Wartosc = 50;
    Gui.WstawKontrolke(&bar_l10K);
    lab_l10K.X1 = 165;
    lab_l10K.Y1 = 0;
    lab_l10K.Tekst = "10K";
    bar_l10K.Id = KL_10K;
    Gui.WstawKontrolke(&lab_l10K);

/*************************************************************** kanał prawy ***********************************************************************/

    tkp.Tekst = "PRAWY";
    tkp.X1 = 0+190;
    tkp.Y1 = 20;
    tkp.Pion = true;
    Gui.WstawKontrolke(&tkp);


    bar_p20.X1 = 10+190;
    bar_p20.X2 = 20+190;
    bar_p20.Y1 = 10;
    bar_p20.Y2 = 63;
    bar_p20.Wartosc = 50;
    bar_p20.Id = KP_20;
    Gui.WstawKontrolke(&bar_p20);
    lab_p20.X1 = 7+190;
    lab_p20.Y1 = 0;
    lab_p20.Tekst = "20";
    Gui.WstawKontrolke(&lab_p20);

    bar_p50.X1 = 30+190;
    bar_p50.X2 = 40+190;
    bar_p50.Y1 = 10;
    bar_p50.Y2 = 63;
    bar_p50.Wartosc = 50;
    bar_p50.Id = KP_50;
    Gui.WstawKontrolke(&bar_p50);
    lab_p50.X1 = 27+190;
    lab_p50.Y1 = 0;
    lab_p50.Tekst = "50";
    Gui.WstawKontrolke(&lab_p50);


    bar_p100.X1 = 50+190;
    bar_p100.X2 = 60+190;
    bar_p100.Y1 = 10;
    bar_p100.Y2 = 63;
    bar_p100.Wartosc = 50;
    bar_p100.Id = KP_100;
    Gui.WstawKontrolke(&bar_p100);
    lab_p100.X1 = 47+190;
    lab_p100.Y1 = 0;
    lab_p100.Tekst = "1D";
    Gui.WstawKontrolke(&lab_p100);


    bar_p200.X1 = 70+190;
    bar_p200.X2 = 80+190;
    bar_p200.Y1 = 10;
    bar_p200.Y2 = 63;
    bar_p200.Wartosc = 50;
    bar_p200.Id = KP_200;
    Gui.WstawKontrolke(&bar_p200);
    lab_p200.X1 = 67+190;
    lab_p200.Y1 = 0;
    lab_p200.Tekst = "2D";
    Gui.WstawKontrolke(&lab_p200);

    bar_p500.X1 = 90+190;
    bar_p500.X2 = 100+190;
    bar_p500.Y1 = 10;
    bar_p500.Y2 = 63;
    bar_p500.Wartosc = 50;
    bar_p500.Id = KP_500;
    Gui.WstawKontrolke(&bar_p500);
    lab_p500.X1 = 87;
    lab_p500.Y1 = 0;
    lab_p500.Tekst = "5D";
    Gui.WstawKontrolke(&lab_p500);

    bar_p1K.X1 = 110+190;
    bar_p1K.X2 = 120+190;
    bar_p1K.Y1 = 10;
    bar_p1K.Y2 = 63;
    bar_p1K.Wartosc = 50;
    bar_p1K.Id = KP_1K;
    Gui.WstawKontrolke(&bar_p1K);
    lab_p1K.X1 = 107+190;
    lab_p1K.Y1 = 0;
    lab_p1K.Tekst = "1K";
    Gui.WstawKontrolke(&lab_p1K);

    bar_p2K.X1 = 130+190;
    bar_p2K.X2 = 140+190;
    bar_p2K.Y1 = 10;
    bar_p2K.Y2 = 63;
    bar_p2K.Wartosc = 50;
    bar_p2K.Id = KP_2K;
    Gui.WstawKontrolke(&bar_p2K);
    lab_p2K.X1 = 127+190;
    lab_p2K.Y1 = 0;
    lab_p2K.Tekst = "2K";
    Gui.WstawKontrolke(&lab_p2K);

    bar_p5K.X1 = 150+190;
    bar_p5K.X2 = 160+190;
    bar_p5K.Y1 = 10;
    bar_p5K.Y2 = 63;
    bar_p5K.Wartosc = 50;
    bar_p5K.Id = KP_5K;
    Gui.WstawKontrolke(&bar_p5K);
    lab_p5K.X1 = 147+190;
    lab_p5K.Y1 = 0;
    lab_p5K.Tekst = "5K";
    Gui.WstawKontrolke(&lab_p5K);

    bar_p10K.X1 = 170+190;
    bar_p10K.X2 = 180+190;
    bar_p10K.Y1 = 10;
    bar_p10K.Y2 = 63;
    bar_p10K.Wartosc = 50;
    Gui.WstawKontrolke(&bar_p10K);
    lab_p10K.X1 = 165+190;
    lab_p10K.Y1 = 0;
    lab_p10K.Tekst = "10K";
    bar_p10K.Id = KP_10K;
    Gui.WstawKontrolke(&lab_l10K);

    radio_eq rad;
    const char *Teksty[] = {"LINIA","MIKROFON"};

    rad.X1 = 190+190;
    rad.Y1 = 20;
    rad.Teksts = Teksty;
    rad.Ilosc = 2;
    rad.Wartosc = 0;
    Gui.WstawKontrolke(&rad);

    Gui.UstawAktywna(KL_20);
    Gui.Aktualizuj();
    Gui.PetlaGlowna();
}

/* FLEXCOMM6_IRQn interrupt handler */
extern "C" void __attribute__((section("RamFunction"))) ObslugaOdczytu(void) {
  /*  Place your code here */
	union {
    	struct{int16_t l,p;};
    	uint32_t dana;
    } zp;

    zp.l = WartoscWY_L;
    zp.p = WartoscWY_P;

    while(!(FLEXCOMM7_PERIPHERAL->FIFOINTSTAT&4));
    while(FLEXCOMM7_PERIPHERAL->FIFOINTSTAT&4)
    	FLEXCOMM7_PERIPHERAL->FIFOWR = zp.dana;


    zp.dana = FLEXCOMM6_PERIPHERAL->FIFORD;

    WartoscWE_L = zp.l;
    WartoscWE_P = zp.p;

    PrzetwarzanieGlowne();

  /* Add for ARM errata 838869, affects Cortex-M4, Cortex-M4F
     Store immediate overlapping exception return operation might vector to incorrect interrupt. */
  #if defined __CORTEX_M && (__CORTEX_M == 4U)
    __DSB();
  #endif
}


int main(void) {

    /* Init board hardware. */
    SYSCON->FMCCR |= SYSCON_FMCCR_FLASHTIM(0b1011);
    BOARD_InitBootClocks();

    CLOCK_EnableClock(kCLOCK_FlexI2s6);
    CLOCK_EnableClock(kCLOCK_FlexI2s7);
    CLOCK_EnableClock(kCLOCK_Sysctl);

    BOARD_InitBootPins();
    BOARD_InitLEDsPins();
    BOARD_InitI2SPins();

    RESET_PeripheralReset(kFC6_RST_SHIFT_RSTn);
    RESET_PeripheralReset(kFC7_RST_SHIFT_RSTn);

    SYSCTL->UPDATELCKOUT = 0;
    /* Set shared signal set 0: SCK, WS from Flexcomm1 */
    SYSCTL->SHAREDCTRLSET[0] |= 0X77;
    /* Set flexcomm3 SCK, WS from shared signal set 0 */
    SYSCTL->FCCTRLSEL[6] |= 0X101;

    BOARD_InitBUTTONsPins();
    BOARD_InitBootPeripherals();

#ifndef BOARD_INIT_DEBUG_CONSOLE_PERIPHERAL
    /* Init FSL debug console. */
    BOARD_InitDebugConsole();
#endif


    LED_BLUE_ON();

    FLEXCOMM7_PERIPHERAL->FIFOSIZE = 4;
    FLEXCOMM6_PERIPHERAL->FIFOSIZE = 4;

    CTIMER0_PERIPHERAL->EMR &= ~(uint32_t)1;
    CTIMER_Reset(CTIMER0_PERIPHERAL);
    CTIMER_StartTimer(CTIMER0_PERIPHERAL);
    while(!(CTIMER0_PERIPHERAL->EMR&1));

    LED_BLUE_OFF();
    BOARD_Codec_Init();

    I2S_Enable(FLEXCOMM7_PERIPHERAL);
    I2S_Enable(FLEXCOMM6_PERIPHERAL);
    NVIC_ClearPendingIRQ(FLEXCOMM6_IRQn);
    NVIC_SetPriority(FLEXCOMM6_IRQn,0);
    FLEXCOMM7_PERIPHERAL->FIFOWR = 0;
    FLEXCOMM7_PERIPHERAL->FIFOWR = 0;
    FLEXCOMM7_PERIPHERAL->FIFOWR = 0;
    FLEXCOMM7_PERIPHERAL->FIFOWR = 0;
    NVIC_EnableIRQ (FLEXCOMM6_IRQn);

    CTIMER0_PERIPHERAL->EMR &= ~(uint32_t)1;
    CTIMER_Reset(CTIMER0_PERIPHERAL);
    CTIMER_StartTimer(CTIMER0_PERIPHERAL);
    while(!(CTIMER0_PERIPHERAL->EMR&1));

    UruchomGUI();

    return 0 ;
}
