/*+*
 * gui.cpp
 *
 *  Created on: 27 sie 2021
 *      Author: Tomek
 */

#include "gui.h"
#include "string.h"
#include "stdlib.h"


EGUI Gui;

EGUI *EGUI::BGui = NULL;

EGUI::EGUI()
{
	BGui = this;

	Przelacznik1 = SFP1;
	Przelacznik2 = SFP2;
	EnkoderWlacznik = SFP3;
	ObslugaEnkodera = SFEN;
	PPrzerwanie = SFPR;
}

int EGUI::ZnajdzKontrolke(int id)
{
	int i;
	for(i=0;i<LicznikKontrolek;++i)
		if(ListaKontrolek[i]->Id==id)
			return i;
	return -1;
}

KontrolkaBaza* EGUI::PodajKontrolke(int id)
{
	int n = ZnajdzKontrolke(id);
	if(n!=-1)
		return this->ListaKontrolek[n];
	return NULL;
}

void EGUI::UsunKontrolke(int id)
{
	int n = ZnajdzKontrolke(id);
	if(n!=-1)
	{
		memcpy(&ListaKontrolek[n], &ListaKontrolek[n+1], LicznikKontrolek-n-1);
		LicznikKontrolek--;
		if(n==LicznikKontrolek)
			AktywnaKontrolka = LicznikKontrolek-1;
		DopasujPozycje();
//		Zmiana = true;
	}
}


void EGUI::UstawAktywna(int id)
{
	int kopia_ak = ZnajdzKontrolke(id);

	if(ListaKontrolek[kopia_ak]->Wybieralny)
		AktywnaKontrolka = kopia_ak;
	DopasujPozycje();
//		Zmiana = true;
}



void EGUI::WstawKontrolke(KontrolkaBaza *k)
{
	ListaKontrolek[LicznikKontrolek++] = k;

}

void EGUI::UsunKontrolki()
{
	LicznikKontrolek = 0;
	AktywnaKontrolka = 0;
	PrzesuniecieX=0;
	PrzesuniecieY=0;
//	Zmiana = true;
}

void EGUI::RysujKontrolke(KontrolkaBaza *k, bool wygaszona, bool aktywna)
{
	int x1,y1,x2,y2;
	if(!k->Bezwzgledny)
	{
		x1 = k->X1 - PrzesuniecieX,
		y1 = k->Y1 - PrzesuniecieY,
		x2 = k->X2 - PrzesuniecieX,
		y2 = k->Y2 - PrzesuniecieY;
	} else
	{
		x1 = k->X1,
		y1 = k->Y1,
		x2 = k->X2,
		y2 = k->Y2;
	}

	switch(k->Typ)
	{
	case K_Bar:
	{
		if(!wygaszona)
		{
			RysujLinie(x1, y1, x2, y1);
			RysujLinie(x2, y1, x2, y2);
			RysujLinie(x1, y1, x1, y2);
			RysujLinie(x1, y2, x2, y2);
		}

		if(!((KBar*)k)->Pozioma)
			RysujProstokat(x1+1, y2-(float)((KBar*)k)->Wartosc*(float)(y2-y1-1)/(float)99, x2-1, y2-1);
		else
			RysujProstokat(x1+1, y1+1, x1+(float)((KBar*)k)->Wartosc*(float)(x2-x1-1)/(float)99, y2-1);

		break;
	}
	case K_CheckBox:
	{
		if(!wygaszona)
		{
			RysujLinie(x1,y1,x1+7,y1);
			RysujLinie(x1,y1+7,x1+7,y1+7);
			RysujLinie(x1,y1,x1,y1+7);
			RysujLinie(x1+7,y1,x1+7,y1+7);
		}
		if(((KCheckBox*)k)->Wartosc)
			RysujProstokat(x1+1, y1+1, x1+6, y1+6);
		RysujTekst(x1+9, y1, ((KCheckBox*)k)->Tekst);
		break;
	}
	case K_Radio:
	{
		int i,y;
		for(i=0, y=y1;i<((KRadio*)k)->Ilosc; ++i, y+=9)
		{
			if(!wygaszona || i != ((KRadio*)k)->Wartosc)
			{
				RysujLinie(x1,y,x1+7,y);
				RysujLinie(x1,y+7,x1+7,y+7);
				RysujLinie(x1,y,x1,y+7);
				RysujLinie(x1+7,y,x1+7,y+7);
			}
			if(i == ((KRadio*)k)->Wartosc)
				RysujProstokat(x1+1, y+1, x1+6, y+6);
			RysujTekst(x1+9, y, ((KRadio*)k)->Teksts[i]);
		}
		break;
	}
	case K_Text:
		RysujTekst(x1+2, y1+2, ((KText*)k)->Tekst);
		if(!wygaszona)
		{
			RysujLinie(x1,y1,x2,y1);
			RysujLinie(x1,y2,x2,y2);
			RysujLinie(x1,y1,x1,y2);
			RysujLinie(x2,y1,x2,y2);
		}
		break;
	case K_Label:
		RysujTekst(x1, y1, ((KLabel*)k)->Tekst, false, ((KLabel*)k)->Pion);
		break;
	case K_Edit:
		if(TrybEdycji)
		{
			if(!wygaszona)
				RysujTekst(x1, y1, ((KEdit*)k)->Tekst);
			else
			{
				char *s = (char*)malloc(strlen(((KEdit*)k)->Tekst));
				strcpy(s,((KEdit*)k)->Tekst);
				s[this->PozycjaEdycji] = ' ';
				RysujTekst(x1, y1, s);
				free(s);
			}
		} else
			RysujTekst(x1, y1, ((KEdit*)k)->Tekst, aktywna);

		break;
	case K_Arrow:
		if(!wygaszona)
		{
			if(y1==y2)
			{
				this->RysujLinie(x1,y1,x2,y1);
				this->RysujLinie(x2-3,y1+3,x2,y1);
				this->RysujLinie(x2-3,y1-3,x2,y1);

			} else if(x1==x2)
			{
				this->RysujLinie(x1,y1,x1,y2);
				this->RysujLinie(x2,y2,x1-3,y2+3);
				this->RysujLinie(x1,y2,x1+3,y2+3);
			} else
			{
				if(((KArrow*)k)->DolnaGorna)
				{
					this->RysujLinie(x1,y1,x2,y1);

					this->RysujLinie(x2,y1,x2,y2);
					this->RysujLinie(x2+3,y2+3,x2,y2);
					this->RysujLinie(x2-3,y2+3,x2,y2);
				} else
				{
					this->RysujLinie(x1,y1,x1,y2);

					this->RysujLinie(x1,y2,x2,y2);
					this->RysujLinie(x2-3,y2+3,x2,y2);
					this->RysujLinie(x2-3,y2-3,x2,y2);
				}
			}
		}
		break;
	}

}


void EGUI::RysujKontrolki()
{
	int i;
	Zmiana = false;
	Wyczysc();
	for(i=0; i<LicznikKontrolek; ++i)
	{
		RysujKontrolke(ListaKontrolek[i], (i==AktywnaKontrolka) ? Wygaszenie : false, i==AktywnaKontrolka);
	}
	AktualizujEkran();
}


bool EGUI::DopasujPozycje()
{
	bool zmiana = false;
	KontrolkaBaza *k = ListaKontrolek[AktywnaKontrolka];
	int wys, szer;

	this->WyliczRozmiar(k, &szer, &wys);

	int xmax = k->X1+szer+5;
	int ymax = k->Y1+wys+5;

	if(xmax-this->PrzesuniecieX >= this->RozmiarX)
		PrzesuniecieX = xmax - RozmiarX+1, zmiana = true;

	if(ymax-this->PrzesuniecieY >= this->RozmiarY)
		PrzesuniecieY = ymax - RozmiarY+1, zmiana = true;

	if(k->X1<PrzesuniecieX+10)
	{
		PrzesuniecieX = k->X1-10;
		if(PrzesuniecieX<0)	PrzesuniecieX = 0;
		zmiana = true;
	}

	if(k->Y1<PrzesuniecieY+10)
	{
		PrzesuniecieY = k->Y1-10;
		if(PrzesuniecieY<0)	PrzesuniecieY = 0;
		zmiana = true;
	}
	return zmiana;
}

void EGUI::ObslugaEnkoder(bool kierunek)
{
	KontrolkaBaza *k = ListaKontrolek[AktywnaKontrolka];
	if(kierunek)
	{
		this->LicznikEnkoder++;
		if(LicznikEnkoder>=3)
		{
			LicznikEnkoder = 0;
			Inkrementacja = true;
		}
	}
	else
	{
		this->LicznikEnkoder --;
		if(LicznikEnkoder<-3)
		{
			LicznikEnkoder = 0;
			Dekrementacja = true;
		}
	}

	switch(k->Typ)
	{
	case K_Bar:
	{
		if(kierunek)
		{
			if (((KBar*)k)->Wartosc<99) ((KBar*)k)->Wartosc++;
		} else
			if(((KBar*)k)->Wartosc>0)	((KBar*)k)->Wartosc--;
		k->CallbackBar(k->Id, kierunek);
		break;
	}
	case K_CheckBox:
		if(Inkrementacja)
		{
			((KCheckBox*)k)->Wartosc = true;
			//k->CallbackCheckBox(k->Id);
		}
		if(Dekrementacja)
		{
			((KCheckBox*)k)->Wartosc = false;
			//k->CallbackCheckBox(k->Id);
		}

		break;
	case K_Radio:
	{
		if(kierunek)
		{
			if(((KRadio*)k)->Wartosc<((KRadio*)k)->Ilosc-1) ++((KRadio*)k)->Wartosc;
		} else
		{
			if(((KRadio*)k)->Wartosc>0) --((KRadio*)k)->Wartosc;
		}

		int szer, wys;
		this->WyliczRozmiar(k, &szer, &wys);

		int xmax = k->X1+szer-1;
		int y = k->Y1 + 9*((KRadio*)k)->Wartosc;
		int ymax = y+8;

		if(xmax-this->PrzesuniecieX >= this->RozmiarX)
			PrzesuniecieX = xmax - RozmiarX+1;

		if(ymax-this->PrzesuniecieY >= this->RozmiarY)
			PrzesuniecieY = ymax - RozmiarY+1;

		if(k->X1<PrzesuniecieX)
		{
			PrzesuniecieX = k->X1;
			if(PrzesuniecieX<0)	PrzesuniecieX = 0;
		}
		if(y<PrzesuniecieY)
		{
			PrzesuniecieY = y;
			if(PrzesuniecieY<0)	PrzesuniecieY = 0;
		}
		k->CallbackRadio(k->Id);
		break;
	}

	case K_Text:
		k->CallbackTekst(k->Id);
		break;
	case K_Label:
		break;
	case K_Edit:
		if(TrybEdycji)
		{
			if(kierunek)
			{
				++ZnakEdycji;
				if(ZnakEdycji>'Z'-' ') ZnakEdycji = 0;
			}
			else
			{
				--ZnakEdycji;
				if(ZnakEdycji<0) ZnakEdycji = 'Z'-' ';
			}
			((KEdit*)k)->Tekst[PozycjaEdycji] = ZnakEdycji+' ';
			k->CallbackEdit(k->Id);
		}
		break;
	case K_Arrow: break;

	}

	Zmiana = true;

	Inkrementacja = false;
	Dekrementacja = false;
}

void EGUI::EnkoderRuch(bool kierunek)
{
	ObslugaEnkoder(kierunek);
}

void EGUI::WyliczRozmiar(KontrolkaBaza *k, int *szerokosc, int *wysokosc)
{
	switch(k->Typ)
	{
	case K_Arrow:
	case K_Bar:
	{
		*szerokosc = k->X2 - k->X1+1;
		*wysokosc = k->Y2-k->Y1+1;
		break;
	}
	case K_CheckBox:
		*szerokosc = 9+strlen(((KCheckBox*)k)->Tekst)*7;
		*wysokosc = 8;
		break;
	case K_Radio:
		int i;
		*szerokosc = 0;
		for(i=0;i<((KRadio*)k)->Ilosc; ++i)
		{
			int l = strlen(((KRadio*)k)->Teksts[i])*7;
			if(l>*szerokosc)	*szerokosc = l;
		}
		*szerokosc += 9;
		*wysokosc = ((KRadio*)k)->Ilosc*9;
		break;
	case K_Text:
		*szerokosc = k->X2 - k->X1+1;
		*wysokosc = k->Y2-k->Y1+1;
		break;
	case K_Label:
		*szerokosc = strlen(((KLabel*)k)->Tekst)*7;
		*wysokosc = 8;
		break;
	case K_Edit:
		*szerokosc = strlen(((KEdit*)k)->Tekst)*7;
		*wysokosc = 8;
		break;
	}
}


void EGUI::ObslugaP2()
{
	KontrolkaBaza *k = this->ListaKontrolek[this->AktywnaKontrolka];
	if(k->Typ == K_Edit)
	{
		TrybEdycji = !TrybEdycji;
		PozycjaEdycji = 0;
		if(TrybEdycji)
		{
			int c = ((KEdit*)k)->Tekst[0];
			ZnakEdycji = c-' ';
		}
	}
	if(k->Typ == K_Text)
		k->CallbackTekst(k->Id);

}

void EGUI::Przycisk1()
{
	if(GlownyPrzycisk != NULL)	GlownyPrzycisk();
}

void EGUI::Przycisk2()
{
	ObslugaP2();
}

void EGUI::ObslugaP3()
{
	if(!TrybEdycji)
	{
		int kopia_ak = AktywnaKontrolka;
		if(kopia_ak<this->LicznikKontrolek-1) ++kopia_ak;

		for(; kopia_ak<this->LicznikKontrolek && !this->ListaKontrolek[kopia_ak]->Wybieralny; ++kopia_ak);
		if(kopia_ak<LicznikKontrolek)
			AktywnaKontrolka = kopia_ak;

		if(this->DopasujPozycje()) Zmiana = true;
	} else
	{
		if(PozycjaEdycji< ((KEdit*)this->ListaKontrolek[this->AktywnaKontrolka])->DlugoscMaks-1)
			++PozycjaEdycji;
	}
}

void EGUI::Przycisk3()
{
	ObslugaP3();
}

void EGUI::ObslugaP4()
{
	if(!TrybEdycji)
	{
		int kopia_ak=AktywnaKontrolka;
		if(kopia_ak>0) --kopia_ak;
		for(; kopia_ak>=0 && !this->ListaKontrolek[kopia_ak]->Wybieralny; --kopia_ak);
		if(kopia_ak>=0)
			AktywnaKontrolka = kopia_ak;
		if(this->DopasujPozycje()) Zmiana = true;
	} else
	{
		if(PozycjaEdycji >0)
			--PozycjaEdycji;
	}
}


void EGUI::Przycisk4()
{
	ObslugaP4();
}

void EGUI::Przerwanie()
{
	static int l_miganie=0;

	if(!BlokowanieWyswietlania)
	{
		if(l_miganie-- == 0)
		{
			Wygaszenie = !Wygaszenie;
			l_miganie = Wygaszenie ? 50 : 75;

			Zmiana = true;
		}
	}
}

void EGUI::PetlaGlowna()
{
	for(;;)
		if(Zmiana)
		{
			this->BlokowaniePrzerwania = true;
			Zmiana = false;
			RysujKontrolki();
			this->BlokowaniePrzerwania = false;
		}
}
