/*
 * gui.h
 *
 *  Created on: 27 sie 2021
 *      Author: Tomek
 */

#ifndef INC_GUI_H_
#define INC_GUI_H_

#define ILOSC_EFEKTOW 100

//#include "przyciski.h"
#include "Grafika.h"
#include "przyciski.h"

class EGUI;

class EGUIDefinicje
{
public:
	enum T_Kontrolka
	{
		K_Bar,
		K_CheckBox,
		K_Radio,
		K_Text,
		K_Label,
		K_Edit,
		K_Arrow
	};
};



class EInterakcjaBaza
{
public:
	virtual void CallbackBar(int id, bool kierunek){};
	virtual void CallbackCheckBox(int id) {};
	virtual void CallbackRadio(int id) {};
	virtual void CallbackTekst(int id) {};
	virtual void CallbackEdit(int id) {};

	virtual void Aktywuj() {}
	virtual void Zamknij() {}


};

class KontrolkaBaza : public EGUIDefinicje, public EInterakcjaBaza
{
public:
	T_Kontrolka Typ;

	int Id;
	int X1,Y1,X2,Y2;
	bool Bezwzgledny = false;
	bool Wybieralny = true;
	friend class EGUI;
};

class KBar : public KontrolkaBaza
{
public:
	KBar() {Typ = K_Bar;}
	int Wartosc;
	bool Pozioma = false;
};

class KCheckBox : public KontrolkaBaza
{
public:
	KCheckBox() { Typ = K_CheckBox; }
	const char *Tekst;
	bool Wartosc = false;
};

class KRadio : public KontrolkaBaza
{
public:
	KRadio() {Typ = K_Radio;}
	const char **Teksts;
	int Wartosc = 0;
	int Ilosc=0;
};

class KText : public KontrolkaBaza
{
public:
	KText() {Typ = K_Text;}
	const char *Tekst;

};

class KLabel : public KontrolkaBaza
{
public:
	KLabel() {Typ=K_Label;Wybieralny = false;}
	const char *Tekst = NULL;
	bool Pion = false;
};


class KEdit : public KontrolkaBaza
{
public:
	KEdit() {Typ=K_Edit;}
	char *Tekst;
	int DlugoscMaks;
};

class KArrow : public KontrolkaBaza
{
public:
	KArrow() {Typ=K_Arrow;}
	bool DolnaGorna = false;
};

class EGUI : public EGUIDefinicje, private Grafika
{
public:
	EGUI();

	friend class EKlientGUI;

	void WstawKontrolke(KontrolkaBaza *);
	void UsunKontrolki();
//	void DodajKontrolke(int);
	void UsunKontrolke(int);
	void UstawAktywna(int);
	int ZnajdzKontrolke(int);
	KontrolkaBaza* PodajKontrolke(int);
	void PetlaGlowna();
	inline void Aktualizuj() {Zmiana = true;}
	void RysujKontrolki();

	void ObslugaEnkoder(bool);
	void ObslugaP2();
	void ObslugaP3();
	void ObslugaP4();

	void (*GlownyPrzycisk)();
protected:

	bool volatile BlokowanieWyswietlania = false;

    bool volatile Zmiana=false;

protected:
	virtual void EnkoderRuch(bool);
	virtual void Przycisk1();
	virtual void Przycisk2();
	virtual void Przycisk3();
	virtual void Przycisk4();

	virtual void Przerwanie();

    int AktywnaKontrolka=0;
    KontrolkaBaza *ListaKontrolek[ILOSC_EFEKTOW];
    int LicznikKontrolek = 0;
	int PrzesuniecieX = 0;
	int PrzesuniecieY = 0;

	void RysujKontrolke(KontrolkaBaza *k, bool, bool);

	void WyliczRozmiar(KontrolkaBaza *k, int*, int*);
	bool DopasujPozycje();

	static EGUI *BGui;
	static void SFP1()	{if(!BGui->BlokowaniePrzerwania) BGui->Przycisk3();}
	static void SFP2()	{if(!BGui->BlokowaniePrzerwania) BGui->Przycisk4();}
	static void SFP3()	{if(!BGui->BlokowaniePrzerwania) BGui->Przycisk2();}
	static void SFEN(bool k)	{if(!BGui->BlokowaniePrzerwania) BGui->EnkoderRuch(k);}
	static void SFPR() {if(!BGui->BlokowaniePrzerwania) BGui->Przerwanie();}

private:

	bool TrybEdycji = false;
	int PozycjaEdycji = 0;
	int ZnakEdycji = 0;
    bool Wygaszenie=false;
    int LicznikEnkoder = 0;
    bool Inkrementacja = false;
    bool Dekrementacja = false;
    bool volatile BlokowaniePrzerwania = false;
};

extern EGUI Gui;

#endif /* INC_GUI_H_ */
